import { Music } from "audio/IAudio";
import { Dictionary } from "language/Dictionaries";
import Translation from "language/Translation";
import Mod from "mod/Mod";
import Register from "mod/ModRegistry";
import Button from "newui/component/Button";
import { CheckButton } from "newui/component/CheckButton";
import Component from "newui/component/Component";

enum DragonQuestMusicTranslation {
	OptionsOnlyDragonQuestMusic,
	OptionsPlayTrackDQM01,
	OptionsPlayTrackDQM02,
	OptionsPlayTrackDQM03,
	OptionsPlayTrackDQM04,
	OptionsPlayTrackDQM05,
	OptionsPlayTrackDQM06,
	OptionsPlayTrackDQM07,
	OptionsPlayTrackDQM08,
	OptionsPlayTrackDQM09,
	OptionsPlayTrackDQM10
}

interface ISaveDataGlobal {
	onlyDragonQuestMusic?: boolean;
}

export default class DragonQuestMusic extends Mod {

	@Register.dictionary("DragonQuestMusic", DragonQuestMusicTranslation)
	public readonly dictionary: Dictionary;

	@Register.musicTrack("DQM01")
	public readonly musicTrackDQM01: Music;
	@Register.musicTrack("DQM02")
	public readonly musicTrackDQM02: Music;
	@Register.musicTrack("DQM03")
	public readonly musicTrackDQM03: Music;
	@Register.musicTrack("DQM04")
	public readonly musicTrackDQM04: Music;
	@Register.musicTrack("DQM05")
	public readonly musicTrackDQM05: Music;
	@Register.musicTrack("DQM06")
	public readonly musicTrackDQM06: Music;
	@Register.musicTrack("DQM07")
	public readonly musicTrackDQM07: Music;
	@Register.musicTrack("DQM08")
	public readonly musicTrackDQM08: Music;
	@Register.musicTrack("DQM09")
	public readonly musicTrackDQM09: Music;
	@Register.musicTrack("DQM10")
	public readonly musicTrackDQM10: Music;

	@Mod.globalData<DragonQuestMusic>("DragonQuest Music")
	public globalData: ISaveDataGlobal;

	private get tracks() {
		return {
                DQM01: this.musicTrackDQM01,
                DQM02: this.musicTrackDQM02,
				DQM03: this.musicTrackDQM03,
				DQM04: this.musicTrackDQM04,
                DQM05: this.musicTrackDQM05,
				DQM06: this.musicTrackDQM06,
				DQM07: this.musicTrackDQM07,
                DQM08: this.musicTrackDQM08,
				DQM09: this.musicTrackDQM09,
				DQM10: this.musicTrackDQM10
		};
	}

	public onInitialize(): any {
		this.refreshMusicHandler(true);
	}

	public onUninitialized() {
		this.resetMusicHandler();
	}

	/**
	 * Undo the music handler changes done by this mod
	 */
	public resetMusicHandler() {
		audio.resetMusicHandler();
		audio.playMusic();
	}

	/**
	 * This method is called in two places:
	 * 1. When initializing (in which case it only does anything if `onlyDragonQuestMusic` is enabled)
	 * 2. When the `onlyDragonQuestMusic` option changes.
	 * 
	 * If `onlyDragonQuestMusic` is enabled, a random track from this mod is played.
	 * If else, the music handler is reset.
	 */
	public refreshMusicHandler(isInitialization = false) {
		if (this.globalData.onlyDragonQuestMusic) {
			audio.getMusicHandler()
				// filter the music tracks to only play the tracks provided by this mod
				.filter(name => name.slice("ModDragonQuestMusic".length) in this.tracks)
				// play a random track
				.moveToRandom();

		} else if (isInitialization) {
			audio.getMusicHandler().refresh();

		} else {
			this.resetMusicHandler();
		}
	}

	/**
	 * Creates an options section for this mod, containing a checkbutton for whether `onlyDragonQuestMusic` 
	 * should be played, and a button to switch to each track.
	 */
	@Register.optionsSection
	public constructOptionsSection(section: Component) {
		// add a checkbutton for whether the music handler should play only alpha tracks
		new CheckButton()
			.setText(() => new Translation(this.dictionary, DragonQuestMusicTranslation.OptionsOnlyDragonQuestMusic))
			.setRefreshMethod(() => !!this.globalData.onlyDragonQuestMusic)
			.event.subscribe("toggle", (_, checked) => {
				this.globalData.onlyDragonQuestMusic = checked;
				this.refreshMusicHandler();
			})
			.appendTo(section);

		// add a button for playing each track
		for (const track of Stream.keys(this.tracks)) {
			new Button()
				.setText(() => new Translation(this.dictionary, `OptionsPlayTrack${track}`))
				.event.subscribe("activate", () => {
					audio.getMusicHandler().moveToEnumEntry(this.tracks[track]);
				})
				.appendTo(section);
		}
	}
}
